import pickle
from typing import Dict
from barfi.scheme.schema_manager import AbsSchemaManager, AbsUserSchemaManager

# Schema Manager
class LocalSchemaManager(AbsSchemaManager):
    def __init__(self, schema_path = None) -> None:
        if schema_path is None:
            schema_path = 'schemas.barfi'
        self.schema_path = schema_path
    
    
    def load_schemas():
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}

        schema_names = list(schemas.keys())
        return {'schema_names': schema_names, 'schemas': schemas}


    def save_schema(schema_name: str, schema_data: Dict):
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}

        with open('schemas.barfi', 'wb') as handle_write:
            schemas[schema_name] = schema_data
            pickle.dump(schemas, handle_write, protocol=pickle.HIGHEST_PROTOCOL)


    def load_schema_name(schema_name: str) -> Dict:
        schemas_barfi = LocalSchemaManager.load_schemas()
        if schema_name in schemas_barfi['schema_names']:
            return schemas_barfi['schemas'][schema_name]
        else:
            raise ValueError(
                f'Schema :{schema_name}: not found in the saved schemas')


    def delete_schema(schema_name: str):
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}

        if schema_name in schemas:
            del schemas[schema_name]
        else:
            raise ValueError(
                f'Schema :{schema_name}: not found in the saved schemas')
        
        with open('schemas.barfi', 'wb') as handle_write:
            pickle.dump(schemas, handle_write, protocol=pickle.HIGHEST_PROTOCOL)


import pickle
from typing import Dict
from barfi.scheme.schema_manager import AbsSchemaManager, AbsUserSchemaManager
from .local_schema_manager import LocalSchemaManager

# User Schema Manager        
class LocalUserSchemaManager(AbsUserSchemaManager):
    def __init__(self, schema_path = None) -> None:
        if schema_path is None:
            schema_path = 'schemas.barfi'
        self.schema_path = schema_path
        
        self.schema_manager = LocalSchemaManager(schema_path)
    
    
    def get_schema_name(schema_name: str, user: str):
        return f'{schema_name}@{user}'
    
    
    def load_schemas(self, user: str):
        schemas = self.schema_manager.load_schemas();
        # 判断schemas是否包含schemas
        if 'schemas' not in schemas:
            return schemas
        
        _all = schemas['schemas']
        if (len(_all) == 0):
            return schemas
        # Dict 按名称过滤
        _filtered = {key: value for key, value in _all.items() if key.endswith(f'@{user}')}
        return {'schema_names': _filtered.keys(), 'schemas': _filtered}
    
    
    def save_schema(self, user: str, schema_data: Dict, schema_name: str):
        self.schema_manager.save_schema(self.get_schema_name(schema_name, user), schema_data)
    
        
    def load_schema_name(self, schema_name: str, user: str) -> Dict:
        return self.schema_manager.load_schema_name(self.get_schema_name(schema_name, user))
    
    
    def delete_schema(self, schema_name: str, user: str):
        self.schema_manager.delete_schema(self.get_schema_name(schema_name, user))


from abc import ABCMeta, abstractmethod
from typing import Dict
 
class AbsSchemaManager(metaclass=ABCMeta):
    @abstractmethod
    def load_schemas(self):
        pass
    
    @abstractmethod
    def save_schema(self, schema_name: str, schema_data: Dict):
        pass
      
    @abstractmethod
    def load_schema_name(schema_name: str) -> Dict:
        pass
    
    @abstractmethod
    def delete_schema(schema_name: str):
        pass
    
    
class AbsUserSchemaManager(AbsSchemaManager):
    @abstractmethod
    def load_schemas(self, user: str):
        pass
    
    @abstractmethod
    def save_schema(self, user: str, schema_data: Dict, schema_name: str):
        pass
      
    @abstractmethod
    def load_schema_name(schema_name: str, user: str) -> Dict:
        pass
    
    @abstractmethod
    def delete_schema(schema_name: str, user: str):
        pass